from django.conf import settings

# Test validator
TEST_STRUCTURE_VALIDATOR_FILE_WHITELIST = []
try:
    TEST_STRUCTURE_VALIDATOR_BASE_DIR = settings.BASE_DIR
except AttributeError:
    TEST_STRUCTURE_VALIDATOR_BASE_DIR = ""
TEST_STRUCTURE_VALIDATOR_BASE_APP_NAME = "apps"
TEST_STRUCTURE_VALIDATOR_APP_LIST = settings.INSTALLED_APPS
TEST_STRUCTURE_VALIDATOR_IGNORED_DIRECTORY_LIST = []
